@@ -1,7 +1,8 @@
 import os
+import numbers
 import numpy as np
 from collections import OrderedDict
-from keras.models import load_model
+from tensorflow.keras.models import load_model
 from pkg_resources import resource_filename
 from distutils.spawn import find_executable
 
@@ -23,8 +24,6 @@ except ImportError:
     print("Warning: You will need to install 'matplotlib' if you wish to plot the classifications.")
 
 
-CLASS_NAMES = ['Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'point-Ia', 'Kilonova', 'SLSN-I',
-               'PISN', 'ILOT', 'CART', 'TDE', 'AGN']
 CLASS_COLOR = {'Pre-explosion': 'grey', 'SNIa-norm': 'tab:green', 'SNIbc': 'tab:orange', 'SNII': 'tab:blue',
                'SNIa-91bg': 'tab:red', 'SNIa-x': 'tab:purple', 'point-Ia': 'tab:brown', 'Kilonova': '#aaffc3',
                'SLSN-I': 'tab:olive', 'PISN': 'tab:cyan', 'ILOT': '#FF1493', 'CART': 'navy', 'TDE': 'tab:pink',
@@ -37,7 +36,8 @@ SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
 
 class Classify(object):
     def __init__(self, known_redshift=True, model_filepath='', passbands=('g', 'r'),
-                 bcut=False, zcut=None, graph=None, model=None):
+                 class_names=('Pre-explosion', 'SNIa-norm', 'SNIbc', 'SNII', 'SNIa-91bg', 'SNIa-x', 'Kilonova', 'SLSN-I', 'TDE'),
+                 nobs=50, mintime=-70, maxtime=80, timestep=3.0, bcut=False, zcut=None, graph=None, model=None):
         """ Takes a list of photometric information and classifies light curves as a function of time
 
         Parameters
@@ -48,6 +48,22 @@ class Classify(object):
             Optional argument. The model is taken from the pre-trained model ZTF model if not specified.
         passbands : tuple
             Optional argument. A tuple listing each passband. E.g. ('g', 'r').
+        class_names : tuple
+            List of class names that the model has been trained on. Note that this must be in the same order
+            as used in training for the model specified in the argument model_filepath.
+            If you are using the default model, leave this argument out.
+        nobs : int
+            Number of points to use in interpolation of light curve between mintime and maxtime.
+            Do not change this argument unless you are using your own model_filepath
+            and have changed this during training.
+        mintime : int
+            Days from trigger (minimum) to extract from light curve.
+        maxtime : int
+            Days from trigger (maximum) to extract from light curve.
+        timestep : float
+            Time-step between interpolated points in light curve.
+            Do not change this argument unless you are using your own model_filepath
+            and have changed this during training.
         bcut : bool
             Cut on galactic latitude.
             Do not set unless you know what you are doing.
@@ -68,10 +84,14 @@ class Classify(object):
         self.passbands = passbands
         self.bcut = bcut
         self.zcut = zcut
-        self.class_names = CLASS_NAMES
+        self.class_names = class_names
+        self.nobs = nobs
+        self.mintime = mintime
+        self.maxtime = maxtime
+        self.timestep = timestep
 
         if self.known_redshift:
-            self.contextual_info = (0,)
+            self.contextual_info = ('redshift',)
             filename = 'keras_model_with_redshift.hdf5'
         else:
             self.contextual_info = ()
@@ -95,7 +115,8 @@ class Classify(object):
     def process_light_curves(self, light_curves):
         processed_lightcurves = read_multiple_light_curves(light_curves, known_redshift=self.known_redshift,
                                                            training_set_parameters=None)
-        prepareinputarrays = PrepareInputArrays(self.passbands, self.contextual_info, self.bcut, self.zcut)
+        prepareinputarrays = PrepareInputArrays(self.passbands, self.contextual_info, self.bcut, self.zcut,
+                                                self.nobs, self.mintime, self.maxtime, self.timestep)
         X, orig_lc, timesX, objids_list, trigger_mjds = prepareinputarrays.prepare_input_arrays(processed_lightcurves)
 
         # # REMOVE CORRECTION FACTOR IF NOT USED
@@ -105,6 +126,31 @@ class Classify(object):
 
         return X, orig_lc, timesX, objids_list, trigger_mjds
 
+    def _do_error_checks(self, light_curves):
+        assert isinstance(light_curves, (list, np.ndarray))
+        for light_curve in light_curves:
+            assert len(light_curve) == 10
+            mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv = light_curve
+            _lcshape = len(mjd)
+            assert _lcshape == len(flux)
+            assert _lcshape == len(fluxerr)
+            assert _lcshape == len(passband)
+            assert _lcshape == len(photflag)
+            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in mjd)
+            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in flux)
+            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in fluxerr)
+            assert all(isinstance(n, numbers.Number) and np.isfinite(n) for n in photflag)
+            assert all(isinstance(n, str) for n in passband)
+            assert isinstance(ra, numbers.Number)
+            assert isinstance(dec, numbers.Number)
+            assert isinstance(mwebv, numbers.Number)
+            assert np.isfinite(ra)
+            assert np.isfinite(dec)
+            assert np.isfinite(mwebv)
+            if self.known_redshift:
+                assert isinstance(redshift, numbers.Number)
+                assert np.isfinite(redshift)
+
     def get_predictions(self, light_curves, return_predictions_at_obstime=False, return_objids=False):
         """ Return the classification accuracies as a function of time for each class
 
@@ -155,23 +201,17 @@ class Classify(object):
             Only provided if return_objids is True.
         """
 
-        # Do error checks
-        assert isinstance(light_curves, (list, np.ndarray))
-        for light_curve in light_curves:
-            assert len(light_curve) == 10
-            mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv = light_curve
-            _lcshape = len(mjd)
-            assert _lcshape == len(flux)
-            assert _lcshape == len(fluxerr)
-            assert _lcshape == len(passband)
-            assert _lcshape == len(photflag)
+        self._do_error_checks(light_curves)
 
         self.X, self.orig_lc, self.timesX, self.objids, self.trigger_mjds = self.process_light_curves(light_curves)
         nobjects = len(self.objids)
 
         if nobjects == 0:
             print("No objects to classify. These may have been removed from the chosen selection cuts")
-            return None, None
+            if return_objids:
+                return None, None, self.objids
+            else:
+                return None, None
 
         if self.graph is not None:
             with self.graph.as_default():
@@ -187,13 +227,16 @@ class Classify(object):
             time_steps = []
             for idx in range(s):
                 obs_time = []
-                for pb in self.passbands:
+                lc_data = self.orig_lc[idx]
+                used_passbands = [pb for pb in self.passbands if pb in lc_data['passband']]
+                for pb in used_passbands:
+                    pbmask = lc_data['passband'] == pb
                     if pb in self.orig_lc[idx]:
-                        obs_time.append(self.orig_lc[idx][pb]['time'].values)
+                        obs_time.append(lc_data[pbmask]['time'].data)
                 obs_time = np.array(obs_time)
                 obs_time = np.sort(obs_time[~np.isnan(obs_time)])
                 y_predict_at_obstime = []
-                for classnum, classname in enumerate(CLASS_NAMES):
+                for classnum, classname in enumerate(self.class_names):
                     y_predict_at_obstime.append(np.interp(obs_time, self.timesX[idx][:argmax[idx]], self.y_predict[idx][:, classnum][:argmax[idx]]))
                 y_predict.append(np.array(y_predict_at_obstime).T)
                 time_steps.append(obs_time + self.trigger_mjds[idx])
@@ -255,22 +298,25 @@ class Classify(object):
             # ax1.axvline(x=0, color='k', linestyle='-', linewidth=1)
             # ax2.axvline(x=0, color='k', linestyle='-', linewidth=1)
 
-            for pbidx, pb in enumerate(self.passbands):
-                if pb in self.orig_lc[idx].keys():
-                    ax1.errorbar(self.orig_lc[idx][pb]['time'], self.orig_lc[idx][pb]['flux'],
-                                 yerr=self.orig_lc[idx][pb]['fluxErr'], fmt=PB_MARKER[pb], label=pb,
-                                 c=PB_COLOR[pb], lw=3, markersize=10, alpha=alpha_observations)
+            lc_data = self.orig_lc[idx]
+            used_passbands = [pb for pb in self.passbands if pb in lc_data['passband']]
+
+            for pbidx, pb in enumerate(used_passbands):
+                pbmask = lc_data['passband'] == pb
+                ax1.errorbar(lc_data[pbmask]['time'], lc_data[pbmask]['flux'],
+                             yerr=lc_data[pbmask]['fluxErr'], fmt=PB_MARKER[pb], label=pb,
+                             c=PB_COLOR[pb], lw=3, markersize=10, alpha=alpha_observations)
                 if plot_matrix_input:
                     ax1.plot(self.timesX[idx][:argmax], self.X[idx][:, pbidx][:argmax], c=PB_COLOR[pb], lw=3)
 
-            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
+            new_t = np.concatenate([lc_data[lc_data['passband'] == pb]['time'].data for pb in used_passbands])
             new_t = np.sort(new_t[~np.isnan(new_t)])
             if not use_interp_flux and not plot_matrix_input:
                 new_y_predict = []
-                for classnum, classname in enumerate(CLASS_NAMES):
+                for classnum, classname in enumerate(self.class_names):
                     new_y_predict.append(np.interp(new_t, self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax]))
 
-            for classnum, classname in enumerate(CLASS_NAMES):
+            for classnum, classname in enumerate(self.class_names):
                 if use_interp_flux or plot_matrix_input:
                     ax2.plot(self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax], '-', label=classname,
                              color=CLASS_COLOR[classname], linewidth=3)
@@ -332,8 +378,11 @@ class Classify(object):
             self.get_predictions(light_curves)
 
         for idx in indexes_to_plot:
-            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
-            all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])
+            lc_data = self.orig_lc[idx]
+            used_passbands = [pb for pb in self.passbands if pb in lc_data['passband']]
+
+            new_t = np.concatenate([lc_data[lc_data['passband'] == pb]['time'].data for pb in used_passbands])
+            all_flux = list(lc_data[lc_data['passband'] == 'g']['flux']) + list(lc_data[lc_data['passband'] == 'r']['flux'])
 
             argmax = self.timesX[idx].argmax() + 1
             fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
@@ -358,11 +407,11 @@ class Classify(object):
             writer = Writer(fps=5, bitrate=1800)
 
             def animate(i):
-                for pbidx, pb in enumerate(self.passbands):
+                for pbidx, pb in enumerate(used_passbands):
                     ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
                              label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])
 
-                for classnum, classname in enumerate(CLASS_NAMES):
+                for classnum, classname in enumerate(self.class_names):
                     ax2.plot(self.timesX[idx][:argmax][:int(i + 1)],
                              self.y_predict[idx][:, classnum][:argmax][:int(i + 1)],
                              '-', label=classname, color=CLASS_COLOR[classname], linewidth=3)
@@ -414,10 +463,13 @@ class Classify(object):
             self.get_predictions()
 
         for idx in indexes_to_plot:
-            new_t = np.array([self.orig_lc[idx][pb]['time'].values for pb in self.passbands]).flatten()
+            lc_data = self.orig_lc[idx]
+            used_passbands = [pb for pb in self.passbands if pb in lc_data['passband']]
+
+            new_t = np.concatenate([lc_data[lc_data['passband'] == pb]['time'].data for pb in used_passbands])
             new_t = np.sort(new_t[~np.isnan(new_t)])
             new_y_predict = []
-            all_flux = list(self.orig_lc[idx]['g']['flux']) + list(self.orig_lc[idx]['r']['flux'])
+            all_flux = list(lc_data[lc_data['passband'] == 'g']['flux']) + list(lc_data[lc_data['passband'] == 'r']['flux'])
 
             argmax = self.timesX[idx].argmax() + 1
             fig, (ax1, ax2) = plt.subplots(nrows=2, ncols=1, figsize=(13, 15),
@@ -441,11 +493,11 @@ class Classify(object):
             Writer = animation.writers['ffmpeg']
             writer = Writer(fps=2, bitrate=1800)
 
-            for classnum, classname in enumerate(CLASS_NAMES):
+            for classnum, classname in enumerate(self.class_names):
                 new_y_predict.append(np.interp(new_t, self.timesX[idx][:argmax], self.y_predict[idx][:, classnum][:argmax]))
 
             def animate(i):
-                for pbidx, pb in enumerate(self.passbands):
+                for pbidx, pb in enumerate(used_passbands):
                     # ax1.plot(self.timesX[idx][:argmax][:int(i + 1)], self.X[idx][:, pbidx][:argmax][:int(i + 1)],
                     #          label=pb, c=PB_COLOR[pb], lw=3)  # , markersize=10, marker=MARKPB[pb])
 
@@ -453,13 +505,14 @@ class Classify(object):
                     if i + 1 >= len(new_t):
                         break
 
-                    dea = [self.orig_lc[idx][pb]['time'] < new_t[int(i+1)]]
+                    pbmask = lc_data['passband'] == pb
+                    dea = [lc_data[pbmask]['time'] < new_t[int(i+1)]]
 
-                    ax1.errorbar(np.array(self.orig_lc[idx][pb]['time'])[dea], np.array(self.orig_lc[idx][pb]['flux'])[dea],
-                                 yerr=np.array(self.orig_lc[idx][pb]['fluxErr'])[dea], fmt=PB_MARKER[pb], label=pb,
+                    ax1.errorbar(np.array(lc_data[pbmask]['time'])[dea], np.array(lc_data[pbmask]['flux'])[dea],
+                                 yerr=np.array(lc_data[pbmask]['fluxErr'])[dea], fmt=PB_MARKER[pb], label=pb,
                                  c=PB_COLOR[pb], lw=3, markersize=10)
 
-                for classnum, classname in enumerate(CLASS_NAMES):
+                for classnum, classname in enumerate(self.class_names):
                     # ax2.plot(self.timesX[idx][:argmax][:int(i + 1)],
                     #          self.y_predict[idx][:, classnum][:argmax][:int(i + 1)],
                     #          '-', label=classname, color=CLASS_COLOR[classname], linewidth=3)
