@@ -36,6 +36,7 @@ class InputLightCurve(object):
             Optional parameter. Cosmological redshift.
         mwebv : float
             Optional parameter. Milky Way E(B-V) extinction.
+            If mwebv is None, then light curve will not be corrected for Milky Way Extinction.
         known_redshift : bool
             Whether to use redshift in processing the light curves and making the arrays.
         training_set_parameters : dict
@@ -120,28 +121,18 @@ class InputLightCurve(object):
 
         laobject = LAobject(locusId=self.objid, objectId=self.objid, time=self.t, flux=self.flux, fluxErr=self.fluxerr,
                             obsId=obsid, passband=self.passband, per=False, mag=False,
-                            photflag=self.photflag, z=self.redshift)
+                            photflag=self.photflag, z=self.redshift, mwebv=self.mwebv)
 
-        outlc = laobject.get_lc(recompute=True)
-
-        otherinfo = [self.redshift, self.b, self.mwebv, self.trigger_mjd]
+        outlc = laobject.get_lc_as_table()
+        outlc.meta = {'redshift': self.redshift, 'b': self.b, 'mwebv': self.mwebv, 'trigger_mjd': self.trigger_mjd}
 
         if self.training_set_parameters is not None:
-            try:
-                t0 = self.compute_t0(outlc)
-            except Exception as e:
-                print(e)
-            otherinfo += [t0, self.peakmjd]
+            t0 = self.compute_t0(outlc)
+            outlc.meta['t0'] = t0
+            outlc.meta['peakmjd'] = self.peakmjd
+            outlc.meta['class_num'] = self.class_number
 
-        savepd = {pb: pd.DataFrame(lcinfo).loc[[0, 3, 4, 7]].rename(
-            {0: 'time', 3: 'flux', 4: 'fluxErr', 7: 'photflag'}).T for pb, lcinfo in
-                  outlc.items()}  # Convert to dataframe rows: time, fluxNorm, fluxNormErr, photFlag; columns: ugrizY
-        savepd['otherinfo'] = pd.DataFrame(otherinfo)
-        savepd = pd.DataFrame(
-            {(outerKey, innerKey): values for outerKey, innerDict in savepd.items() for innerKey, values in
-             innerDict.items()})  # Use multilevel indexing
-
-        return savepd
+        return outlc
 
 
 def read_multiple_light_curves(light_curve_list, known_redshift=True, training_set_parameters=None):
@@ -155,7 +146,8 @@ def read_multiple_light_curves(light_curve_list, known_redshift=True, training_s
     processed_light_curves = {}
     for light_curve in light_curve_list:
         mjd, flux, fluxerr, passband, photflag, ra, dec, objid, redshift, mwebv = light_curve
-        inputlightcurve = InputLightCurve(*light_curve, known_redshift, training_set_parameters)
+        inputlightcurve = InputLightCurve(*light_curve, known_redshift=known_redshift,
+                                          training_set_parameters=training_set_parameters)
         processed_light_curves[objid] = inputlightcurve.preprocess_light_curve()
 
     return processed_light_curves
